-- Average Enjoyer ^ω^ Lampovo Project
-- vk.com/lampovofs

licensePlatesStandalone = {}
licensePlatesStandalone.modDir = g_currentModDirectory

LicensePlateManager.loadMapData = Utils.overwrittenFunction(LicensePlateManager.loadMapData, function (self, superFunc, xmlFile, missionInfo, baseDirectory)
    local xmlFilename = Utils.getFilename('data/fonts.xml', licensePlatesStandalone.modDir)
    g_materialManager:loadFontMaterialsXML(xmlFilename, nil, licensePlatesStandalone.modDir)

    LicensePlateManager:superClass().loadMapData(self, xmlFile, missionInfo, baseDirectory)

    LicensePlateManager.createLicensePlateXMLSchema()

    local filename = getXMLString(xmlFile, "map.licensePlates#filename")
    if filename ~= nil then
        self.xmlFilename = Utils.getFilename('data/licensePlates.xml', licensePlatesStandalone.modDir)
        self.licensePlateXML = XMLFile.load("mapLicensePlates", self.xmlFilename, LicensePlateManager.xmlSchema)
        if self.licensePlateXML ~= nil then
            self.xmlReferences = 0
            self:loadLicensePlatesFromXML(self.licensePlateXML, licensePlatesStandalone.modDir)

            if self.licensePlateXML ~= nil and self.xmlReferences == 0 then
                self.licensePlateXML:delete()
                self.licensePlateXML = nil
            end
        end
    end

    return true
end)

LicensePlateDialog.updateVariations = Utils.overwrittenFunction(LicensePlateDialog.updateVariations, function (self, superFunc)
    local texts = {}

    for i = 1, #self.licensePlate.variations do
        local typeIndex

        if i == 1 or i == 2 then
            typeIndex = 1
        elseif i == 3 or i == 4 then
            typeIndex = 2
        elseif i == 5 or i == 6 then
            typeIndex = 3
        else
            typeIndex = i
        end

        local typeText = string.format(g_i18n:getText("ui_licensePlateTypeItem"), typeIndex) .. " " .. g_i18n:getText("ui_licensePlateType" .. i)
        table.insert(texts, typeText)
    end

    self.typeOption:setTexts(texts)
    self.typeOption:setState(self.currentVariation)
end)

LicensePlate.setVariation = Utils.overwrittenFunction(LicensePlate.setVariation, function (self, superFunc, variationIndex)
    local variation = self.variations[variationIndex]
    if variation ~= nil then
        local oldVariation = self.variations[self.variationIndex]
        for i=1, #oldVariation.values do
            local value = oldVariation.values[i]
            if not value.isStatic then
                for j=1, getNumOfChildren(value.node) do
                    delete(getChildAt(value.node, j - 1))
                end
            end
        end

        self.variationIndex = variationIndex

        local dimensions = {
            [3] = {width = 0.47, fontSize = 0.072},
            [4] = {width = 0.47, fontSize = 0.072},
            [5] = {width = 0.303, height = 0.085, fontSize = 0.052},
            [6] = {width = 0.303, height = 0.085, fontSize = 0.052},
        }

        local defaultValues = {
            elongated = {width = 0.52, height = 0.11, fontSize = 0.072},
            regular = {width = 0.34, height = 0.21, fontSize = 0.07},
            variation5or6 = {width = 0.22, height = 0.152, fontSize = 0.06},
        }

        local isFontScaleX95 = (variationIndex == 2 or variationIndex == 4 or variationIndex == 6)

        if self.type == LicensePlateManager.PLATE_TYPE.ELONGATED then
            local dim = dimensions[variationIndex]
            if dim then
                self.width = dim.width
                self.height = dim.height or self.height
                self.fontSize = dim.fontSize
            else
                self.width = defaultValues.elongated.width
                self.height = defaultValues.elongated.height
                self.fontSize = defaultValues.elongated.fontSize
            end
        else
            if variationIndex == 5 or variationIndex == 6 then
                self.width = defaultValues.variation5or6.width
                self.height = defaultValues.variation5or6.height
                self.fontSize = defaultValues.variation5or6.fontSize
            else
                self.width = defaultValues.regular.width
                self.height = defaultValues.regular.height
                self.fontSize = defaultValues.regular.fontSize
            end
        end

        self.fontScaleX = isFontScaleX95 and 1.2 or 0.95

        for i=1, #variation.values do
            local value = variation.values[i]

            if not value.isStatic then
                value.characterLine = CharacterLine.new(value.node, self.font, 1)
                value.characterLine:setSizeAndScale(self.fontSize, self.fontScaleX, self.fontScaleY)
                value.characterLine:setTextAlignment(RenderText.ALIGN_CENTER)
                value.characterLine:setTextVerticalAlignment(RenderText.VERTICAL_ALIGN_MIDDLE)
            end
        end

        for i=1, #variation.materials do
            local material = variation.materials[i]
            material:apply(self.node)
        end
    end
end)